'use strict';

exports.__esModule = true;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

exports.makeTree = makeTree;
exports.filterRedundantErrors = filterRedundantErrors;
exports.createErrorInstances = createErrorInstances;

var _utils = require('./utils');

var _validationErrors = require('./validation-errors');

var _debug = require('./debug');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var JSON_POINTERS_REGEX = /\/\w+(\/\d+)?/g;

// Make a tree of errors from ajv errors array
function makeTree() {
  var ajvErrors = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  var root = { children: {} };
  ajvErrors.forEach(function (ajvError) {
    var dataPath = ajvError.dataPath,
        keyword = ajvError.keyword;

    // `dataPath === ''` is root

    var paths = dataPath === '' ? [''] : dataPath.match(JSON_POINTERS_REGEX);
    paths && paths.reduce(function (obj, path, i) {
      obj.children[path] = obj.children[path] || { children: {}, errors: [] };
      if (i === paths.length - 1) {
        obj.children[path].errors.push(ajvError);
      }
      return obj.children[path];
    }, root);
  });
  return root;
}

function filterRedundantErrors(root, parent, key) {
  /**
   * If there is an `anyOf` error that means we have more meaningful errors
   * inside children. So we will just remove all errors from this level.
   */
  if ((0, _utils.getErrors)(root).some(_utils.isAnyOfError)) {
    root.errors = undefined;
  }

  /**
   * If all errors are `enum` and siblings have any error then we can safely
   * ignore the node.
   *
   * **CAUTION**
   * Need explicit `root.errors` check because `[].every(fn) === true`
   * https://en.wikipedia.org/wiki/Vacuous_truth#Vacuous_truths_in_mathematics
   */
  if (root.errors && (0, _utils.getErrors)(root).every(_utils.isEnumError)) {
    if ((0, _utils.getSiblings)(parent)(root)
    // Remove any reference which becomes `undefined` later
    .filter(_utils.notUndefined).some(_utils.getErrors)) {
      parent.children[key] = undefined;
    }
  }

  (0, _entries2.default)(root.children).forEach(function (_ref) {
    var key = _ref[0],
        child = _ref[1];
    return filterRedundantErrors(child, root, key);
  });
}

function createErrorInstances(root, indent) {
  var errors = (0, _utils.getErrors)(root);
  if (errors.length && errors.every(_utils.isEnumError)) {
    var allowedValues = (0, _utils.concatAll)([])(errors.map(function (e) {
      return e.params.allowedValues;
    }));
    var error = errors[0];
    return [new _validationErrors.EnumValidationError((0, _assign2.default)({}, error, {
      params: { allowedValues }
    }), indent)];
  } else {
    return (0, _utils.concatAll)(errors.reduce(function (ret, error) {
      switch (error.keyword) {
        case 'additionalProperties':
          return ret.concat(new _validationErrors.AdditionalPropValidationError(error, indent));
        case 'required':
          return ret.concat(new _validationErrors.RequiredValidationError(error, indent));
        default:
          return ret.concat(new _validationErrors.DefaultValidationError(error, indent));
      }
    }, []))((0, _utils.getChildren)(root).map(function (child) {
      return createErrorInstances(child, indent);
    }));
  }
}

exports.default = function (ajvErrors, indent) {
  var tree = makeTree(ajvErrors || []);
  // debug(JSON.stringify(tree));
  filterRedundantErrors(tree);
  var errors = createErrorInstances(tree, indent);
  if (!errors) {
    (0, _debug.debug)((0, _stringify2.default)(tree));
  }
  return errors;
};