'use strict';

exports.__esModule = true;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _leven = require('leven');

var _leven2 = _interopRequireDefault(_leven);

var _jsonpointer = require('jsonpointer');

var _jsonpointer2 = _interopRequireDefault(_jsonpointer);

var _print = require('../json/print');

var _print2 = _interopRequireDefault(_print);

var _findErrorPosition2 = require('../json/find-error-position');

var _findErrorPosition3 = _interopRequireDefault(_findErrorPosition2);

var _base = require('./base');

var _base2 = _interopRequireDefault(_base);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var EnumValidationError = function (_BaseValidationError) {
  (0, _inherits3.default)(EnumValidationError, _BaseValidationError);

  function EnumValidationError() {
    (0, _classCallCheck3.default)(this, EnumValidationError);
    return (0, _possibleConstructorReturn3.default)(this, _BaseValidationError.apply(this, arguments));
  }

  EnumValidationError.prototype.print = function print(schema, data) {
    var output = [];
    var _options = this.options,
        message = _options.message,
        dataPath = _options.dataPath,
        allowedValues = _options.params.allowedValues;

    var bestMatch = this.findBestMatch(data);

    output.push(_chalk2.default`{red {bold ENUM} ${message}}`);
    output.push(_chalk2.default`{red (${allowedValues.join(', ')})}\n`);

    return output.concat((0, _print2.default)(data, dataPath, { indent: this.indent })(function () {
      if (bestMatch !== null) {
        return _chalk2.default`☝🏽  Did you mean {bold ${bestMatch}} here?`;
      } else {
        return _chalk2.default`☝🏽  Unexpected value here`;
      }
    }));
  };

  EnumValidationError.prototype.getError = function getError(schema, data) {
    var _options2 = this.options,
        keyword = _options2.keyword,
        message = _options2.message,
        dataPath = _options2.dataPath,
        params = _options2.params;

    var jsonString = (0, _stringify2.default)(data, null, this.indent);

    var _findErrorPosition = (0, _findErrorPosition3.default)(jsonString, dataPath),
        line = _findErrorPosition.line,
        column = _findErrorPosition.column;

    var bestMatch = this.findBestMatch(data);

    var output = {
      line,
      column,
      error: `${dataPath} ${message}: ${params.allowedValues.join(', ')}`
    };

    if (bestMatch !== null) {
      output.suggestion = `Did you mean ${bestMatch}?`;
    }

    return output;
  };

  EnumValidationError.prototype.findBestMatch = function findBestMatch(data) {
    var _options3 = this.options,
        dataPath = _options3.dataPath,
        allowedValues = _options3.params.allowedValues;

    var currentValue = _jsonpointer2.default.get(data, dataPath);

    var bestMatch = allowedValues.map(function (value) {
      return {
        value,
        weight: (0, _leven2.default)(value, currentValue.toString())
      };
    }).sort(function (x, y) {
      return x.weight > y.weight ? 1 : x.weight < y.weight ? -1 : 0;
    })[0];

    return allowedValues.length === 1 || bestMatch.weight < 5 ? bestMatch.value : null;
  };

  return EnumValidationError;
}(_base2.default);

exports.default = EnumValidationError;
module.exports = exports['default'];