'use strict';

exports.__esModule = true;

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _findErrorPosition = require('./find-error-position');

var _findErrorPosition2 = _interopRequireDefault(_findErrorPosition);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = (obj, dataPath, options = {}) => annotate => {
  const {
    delta = 5,
    defaultColor = 'gray',
    errorColor = 'red',
    annotationColor = 'cyan',
    gutterChar = '│',
    annotationGutterChar = '〉',
    indent = 2
  } = options;

  try {
    const jsonString = JSON.stringify(obj, null, indent);
    const { line, column } = (0, _findErrorPosition2.default)(jsonString, dataPath);

    const lines = jsonString.split('\n');
    const gutterWidth = lines.length.toString().length;
    const annotationGutter = `${''.padStart(gutterWidth)}${annotationGutterChar}`;

    const min = Math.max(line - delta, 0);
    const max = Math.min(line + delta, lines.length - 1);
    let output = [];

    for (let i = min; i <= max; ++i) {
      const color = i === line ? errorColor : defaultColor;
      output.push(_chalk2.default`{${color} ${String(i + 1).padStart(gutterWidth)}${gutterChar} ${lines[i]}}`);
      if (i === line) {
        output = output.concat([
        // New line
        _chalk2.default`{${annotationColor} ${annotationGutter}}`, _chalk2.default`{${annotationColor} ${annotationGutter}${''.padStart(column)}${annotate()}}`,

        // New line
        _chalk2.default`{${annotationColor} ${annotationGutter}}`]);
      }
    }
    return output;
  } catch (e) {
    return [_chalk2.default`{red 😢  ${e.message || 'Error'}}`];
  }
};

module.exports = exports['default'];