'use strict';

exports.__esModule = true;

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _leven = require('leven');

var _leven2 = _interopRequireDefault(_leven);

var _jsonpointer = require('jsonpointer');

var _jsonpointer2 = _interopRequireDefault(_jsonpointer);

var _print = require('../json/print');

var _print2 = _interopRequireDefault(_print);

var _findErrorPosition = require('../json/find-error-position');

var _findErrorPosition2 = _interopRequireDefault(_findErrorPosition);

var _base = require('./base');

var _base2 = _interopRequireDefault(_base);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class EnumValidationError extends _base2.default {
  print(schema, data) {
    const output = [];
    const { message, dataPath, params: { allowedValues } } = this.options;
    const bestMatch = this.findBestMatch(data);

    output.push(_chalk2.default`{red {bold ENUM} ${message}}`);
    output.push(_chalk2.default`{red (${allowedValues.join(', ')})}\n`);

    return output.concat((0, _print2.default)(data, dataPath, { indent: this.indent })(() => {
      if (bestMatch !== null) {
        return _chalk2.default`☝🏽  Did you mean {bold ${bestMatch}} here?`;
      } else {
        return _chalk2.default`☝🏽  Unexpected value here`;
      }
    }));
  }

  getError(schema, data) {
    const { keyword, message, dataPath, params } = this.options;
    const jsonString = JSON.stringify(data, null, this.indent);
    const { line, column } = (0, _findErrorPosition2.default)(jsonString, dataPath);
    const bestMatch = this.findBestMatch(data);

    const output = {
      line,
      column,
      error: `${dataPath} ${message}: ${params.allowedValues.join(', ')}`
    };

    if (bestMatch !== null) {
      output.suggestion = `Did you mean ${bestMatch}?`;
    }

    return output;
  }

  findBestMatch(data) {
    const { dataPath, params: { allowedValues } } = this.options;
    const currentValue = _jsonpointer2.default.get(data, dataPath);

    const bestMatch = allowedValues.map(value => ({
      value,
      weight: (0, _leven2.default)(value, currentValue.toString())
    })).sort((x, y) => x.weight > y.weight ? 1 : x.weight < y.weight ? -1 : 0)[0];

    return allowedValues.length === 1 || bestMatch.weight < 5 ? bestMatch.value : null;
  }
}
exports.default = EnumValidationError;
module.exports = exports['default'];